////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
//
// XPS Printer Driver SDK: emboss + indent + hopper selection c++ sample.
////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <bidispl.h>
#include <WinSpool.h>
#include <iostream>
#include <sstream>
#include "XGetopt.h"
#include "util.h"
#include "DXP01SDK.H"

using namespace std;

void usage() {
    const string shortExeName = CT2A(util::GetShortExeName());
    cout << endl << shortExeName << " demonstrates the emboss and indent escapes, input hopper" << endl;
    cout << "selection, and the checking of print ribbon and emboss supplies." << endl << endl;
    cout << "Emboss and indent data is hardcoded." << endl << endl;
    cout << shortExeName << " -n <printername> [-i <input hopper>] [-s] [-c]" << endl << endl;
    cout << "options:" << endl;
    cout << "  " << "-n <printername>. Required. Try -n \"XPS Card Printer\"." << endl;
    cout << "  " << "-i <input hopper>. Optional. Defaults to \"1\"." << endl;
    cout << "  " << "-s Check Supplies during StartJob. Optional. Defaults to false." << endl;
    cout << "  " << "-c Poll for job completion." << endl;
    exit(0);
}

////////////////////////////////////////////////////////////////////////////////
// SendEmbossIndentData()
// use escapes in a windows print job document to emboss and indent a card.
////////////////////////////////////////////////////////////////////////////////
void SendEmbossIndentData(CString printerName)
{
    stringstream exceptionText;
    HANDLE printerHandle(0);
    HDC printerDC(0);

    try {

        int rc = ::OpenPrinter(printerName.GetBuffer(), &printerHandle, NULL);
        if (!rc) {
            const DWORD lastErr = ::GetLastError();
            exceptionText << "OpenPrinter():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
            throw runtime_error(exceptionText.str());
        }

        printerDC = ::CreateDC(L"WINSPOOL", printerName, NULL, NULL);
        if (NULL == printerDC) {
            const DWORD lastErr = ::GetLastError();
            exceptionText << "CreateDC():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
            throw runtime_error(exceptionText.str());
        }

        ::DOCINFO docInfo = {0};
        docInfo.cbSize = sizeof DOCINFO;
        docInfo.lpszDocName = L"emboss indent job";
        rc = ::StartDoc(printerDC, &docInfo);
        if (rc < 0) {
            const DWORD lastErr = ::GetLastError();
            exceptionText << "StartDoc():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
            throw runtime_error(exceptionText.str());
        }

        const DWORD windowsJobID = rc;

        rc = ::StartPage(printerDC);
        if (rc <= 0) {
            const DWORD lastErr = ::GetLastError();
            exceptionText << "StartPage():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
            throw runtime_error(exceptionText.str());
        }

        CString text;

        // emit all emboss + indent data driver escape strings:
        {
            text = L"~EM%1;301;860;Font 11111";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%2;1600;860;222222";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%3;301;1460;333333";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%4;301;1180;444444";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%5;301;690;555555";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%6;1600;690;666666";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%7;301;650;777777";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%8;301;1000;888888";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%9;301;1050;999999";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
            text = L"~EM%10;1600;1050;10 10 10";
            rc = ::TextOut(printerDC, 50, 50, text, text.GetLength());
            if (0 == rc) {
                const DWORD lastErr = ::GetLastError();
                exceptionText << "TextOut():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
                throw runtime_error(exceptionText.str());
            }
        }

        rc = ::EndPage(printerDC);
        if (rc <= 0) {
            const DWORD lastErr = ::GetLastError();
            exceptionText << "EndPage():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
            throw runtime_error(exceptionText.str());
        }

        rc = ::EndDoc(printerDC);
        if (rc < 0) {
            const DWORD lastErr = ::GetLastError();
            exceptionText << "EndDoc():" << " 0x" << hex << lastErr << " " << CT2A(util::Win32ErrorString(lastErr));
            throw runtime_error(exceptionText.str());
        }

        util::WaitUntilJobSpooled(printerHandle, windowsJobID);
    }
    catch (runtime_error&) {
        if (0 != printerDC) {
            ::DeleteDC(printerDC);
        }
        if (0 != printerHandle) {
            ::ClosePrinter(printerHandle);
        }
        throw;
    }

    if (0 != printerDC) {
        ::DeleteDC(printerDC);
    }

    if (0 != printerHandle) {
        ::ClosePrinter(printerHandle);
    }
}

////////////////////////////////////////////////////////////////////////////////
// main()
////////////////////////////////////////////////////////////////////////////////
int _tmain(int argc, _TCHAR* argv[])
{
    stringstream exceptionText;
    CString printerName;
    CString inputHopper(TEXT("1"));
    bool checkSupplies(false);
    bool pollForJobCompletion(false);
    int c(0);

    while ((c = getopt(argc, argv, _T("n:ci:s"))) != EOF)
    {
        switch (c)
        {
        case L'n': printerName = optarg; break;
        case L'i': inputHopper = optarg; break;
        case L'c': pollForJobCompletion = true; break;
        case L's': checkSupplies = true; break;
        default:
            usage();
        }
    }

    if (printerName.IsEmpty()) {
        cout << "printername required." << endl;
        usage();
    }

    if (inputHopper.IsEmpty()) {
        cout << "inputHopper required." << endl;
        usage();
    }

    CComPtr <IBidiSpl> bidiSpl;
    int printerJobID(0);

    try {
        HRESULT hr = ::CoInitialize(NULL);
        if (FAILED(hr)) {
            exceptionText << "::CoInitialize:" << " 0x" << hex << hr << " " << CT2A(util::Win32ErrorString(hr));
            throw runtime_error(exceptionText.str());
        }

        // create an IBidiSpl COM smartpointer instance and bind it to the printer
        // name. We need it for job completion status - not embossing or indenting.
        hr = bidiSpl.CoCreateInstance(CLSID_BidiSpl);
        if (FAILED(hr)) {
            exceptionText << "bidiSpl.CoCreateInstance:" << " 0x" << hex << hr << " " << CT2A(util::Win32ErrorString(hr));
            throw runtime_error(exceptionText.str());
        }

        hr = bidiSpl->BindDevice(printerName, BIDI_ACCESS_USER);
        if (FAILED(hr)) {
            exceptionText << "BindDevice():" << " 0x" << hex << hr << " " << CT2A(printerName) << " " << CT2A(util::Win32ErrorString(hr));
            throw runtime_error(exceptionText.str());
        }

        const CString driverVersionXml = util::GetDriverVersionXML(bidiSpl);
        const CString driverVersionString = util::ParseDriverVersionXML(driverVersionXml);
        cout << endl << "driver version: " << CW2A(driverVersionString) << endl << endl;

        // is the printer is in the ready state?
        const CString printerOptionsXML = util::GetPrinterOptionsXML(bidiSpl);
        util::PrinterOptionsValues printerOptionsValues = util::ParsePrinterOptionsXML(printerOptionsXML);

        if ("Installed" != printerOptionsValues._moduleEmbosser) {
            exceptionText << CT2A(printerName) << ": embosser is not present.";
            throw runtime_error(exceptionText.str());
        }

        if ("Ready" != printerOptionsValues._printerStatus && "Busy" != printerOptionsValues._printerStatus) {
            exceptionText << "printer " << CT2A(printerName) << " is not ready.";
            throw runtime_error(exceptionText.str());
        }

        printerJobID = util::StartJob(
            bidiSpl,
            checkSupplies,
            inputHopper);

        if (0 == printerJobID) {
            exceptionText << "StartJob() failed.";
            throw runtime_error(exceptionText.str());
        }

        SendEmbossIndentData(printerName);

        util::EndJob(bidiSpl);

        if (pollForJobCompletion) {
            util::PollForJobCompletion(bidiSpl, printerJobID);
        }
    }
    catch (util::BidiException& e) {
        cerr << e.what() << endl;
        util::CancelJob(bidiSpl, e.printerJobID, e.errorCode);
    }
    catch (runtime_error& e) {
        cerr << e.what() << endl;
        if (0 != printerJobID) {
            util::CancelJob(bidiSpl, printerJobID, 0);
        }
    }

    if (bidiSpl) {
        bidiSpl->UnbindDevice();
        bidiSpl = NULL;
    }

    ::CoUninitialize();

    return 0;
}
