﻿////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
//
// XPS Printer SDK: csharp 'printer_control' sample
////////////////////////////////////////////////////////////////////////////////
using System;
using System.IO;
using dxp01sdk;

public enum Commands {
    undefined,
    cancelAllJobs,
    restart,
    resetCardCounts
}

public class CommandLineOptions {
    public CommandLineOptions() { command = Commands.undefined; }
    public string printerName;
    public Commands command;
}

class control {
    private static void Usage() {
        string thisExeName = Path.GetFileName(Environment.GetCommandLineArgs()[0]);
        Console.WriteLine();
        Console.WriteLine(thisExeName + " controls the printer through the printer driver.");
        Console.WriteLine();
        Console.WriteLine(thisExeName + " -n <printername> <command>");
        Console.WriteLine("  -n <printername>. Required. Try -n \"XPS Card Printer\".");
        Console.WriteLine("  -c cancel all jobs: Send a command to the printer to clear any print job");
        Console.WriteLine("                    from the printer. This does not delete spooled Windows");
        Console.WriteLine("                    print jobs.");
        Console.WriteLine("  -r restart the printer.");
        Console.WriteLine("  -e reset all card counts.");
        Console.WriteLine();
        Environment.Exit(-1);
    }

    static CommandLineOptions GetCommandlineOptions(string[] args) {
        CommandLineOptions commandLineOptions = new CommandLineOptions();
        CommandLine.Utility.Arguments arguments = new CommandLine.Utility.Arguments(args);

        if (string.IsNullOrEmpty(arguments["n"])) {
            Usage();
        }

        // check for -n with no printer name:
        bool boolVal = false;
        if (Boolean.TryParse(arguments["n"], out boolVal)) {
            Usage();
        }
        commandLineOptions.printerName = arguments["n"];

        if (!string.IsNullOrEmpty(arguments["c"])) {
            commandLineOptions.command = Commands.cancelAllJobs;
        }

        if (!string.IsNullOrEmpty(arguments["r"])) {
            commandLineOptions.command = Commands.restart;
        }

        if (!string.IsNullOrEmpty(arguments["e"])) {
            commandLineOptions.command = Commands.resetCardCounts;
        }

        if (Commands.undefined == commandLineOptions.command) {
            Usage();
        }

        return commandLineOptions;
    }

    ////////////////////////////////////////////////////////////////////////////
    // Main()
    ////////////////////////////////////////////////////////////////////////////
    static void Main(string[] args) {

        CommandLineOptions commandLineOptions = GetCommandlineOptions(args);

        BidiSplWrap bidiSpl = new BidiSplWrap();
        bidiSpl.BindDevice(commandLineOptions.printerName);

        switch (commandLineOptions.command) {
            case Commands.cancelAllJobs:
                // using zero for the printerJobID and the errorCode causes all
                // printerJobs to be canceled:
                Util.CancelJob(bidiSpl, 0, 0);
                Console.WriteLine("all jobs canceled for '{0}'", commandLineOptions.printerName);
                break;

            case Commands.restart:
                bidiSpl.SetPrinterData(strings.RESTART_PRINTER);
                Console.WriteLine("command issued to restart the printer.");
                break;

            case Commands.resetCardCounts:
                bidiSpl.SetPrinterData(strings.RESET_CARD_COUNTS);
                Console.WriteLine("command issued to reset the printer's card counts.");
                break;

            default:
                Console.WriteLine("unexpected commandline option.");
                break;
        }

        bidiSpl.UnbindDevice();
    }
}