////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
////////////////////////////////////////////////////////////////////////////////
package com.datacard.xpsprinter.sdk.examples;

import java.io.IOException;
import com.sun.jna.examples.XPS_Java_SDK;
import com.sun.jna.examples.XPS_Java_SDK.XpsDriverInteropLib;

public class SmartCard {

    private final String mPrinterName;

    public SmartCard(String aPrinterName) {
        mPrinterName = aPrinterName;
    }

    public boolean DemoSmartCard(
            boolean doMagstripeEncode,
            boolean bDoPrint,
            boolean isLandscape,
            boolean isDuplex,
            boolean isSingleWire,
            boolean parkBack) {
        final int S_OK = 0;
        byte returnXML[] = new byte[XPS_Java_SDK.BUFFSIZE];
        int sizeOfReturnXML[] = new int[1];
        sizeOfReturnXML[0] = XPS_Java_SDK.BUFFSIZE;
        boolean bDisplayError = true;
        boolean bJobStarted = false;
        boolean bJobHasBeenCanceled = false;
        int printerJobID;

        PrinterStatusXML printerStatusXml = new PrinterStatusXML();

        int hResult = XpsDriverInteropLib.INSTANCE.StartJob(mPrinterName, returnXML, sizeOfReturnXML);
        if (S_OK == hResult) {
            bJobStarted = true;
            sizeOfReturnXML[0] = XPS_Java_SDK.BUFFSIZE;
            if (S_OK == XpsDriverInteropLib.INSTANCE.SmartCardPark(
                    mPrinterName,
                    returnXML,
                    sizeOfReturnXML,
                    parkBack)) {

                // we have parked the card in the smartcard reader.
                printerStatusXml.Parse(returnXML, sizeOfReturnXML);
                printerJobID = printerStatusXml.GetPrintJobID();
                System.out.format("smartcard PARK succeed. PrintJobID: %s\n", printerJobID);

                // simulate smartcard personalization:
                SmartCard_singlewire singleWireDemo = new SmartCard_singlewire(mPrinterName);

                if (isSingleWire) {
                    System.out.println();
                    System.out.println("Demo singlewire contact smartcard");
                    singleWireDemo.DemoSmartCard_Singlewire(
                            doMagstripeEncode,
                            bDoPrint,
                            isLandscape,
                            isDuplex,
                            SmartCard_singlewire.ChipConnection_Contact);
                    System.out.println();
                    System.out.println("Demo singlewire contactless smartcard");
                    singleWireDemo.DemoSmartCard_Singlewire(
                            doMagstripeEncode,
                            bDoPrint,
                            isLandscape,
                            isDuplex,
                            SmartCard_singlewire.ChipConnection_Contactless);
                }

                byte smartcardUserResponse[] = new byte[2];
                try {
                    Thread.sleep(2000);
                    System.out.println("Press Enter to resume or 'c' to cancel the printjob: ");
                    System.in.read(smartcardUserResponse, 0, 1);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                sizeOfReturnXML[0] = XPS_Java_SDK.BUFFSIZE;

                if (smartcardUserResponse[0] != 'c') {
                    System.out.println("Smartcard personalization succeeded. Resuming print job.");
                    if (XpsDriverInteropLib.INSTANCE.ResumeJob(mPrinterName,
                            printerStatusXml.GetPrintJobID(), printerStatusXml.GetErrorCode(), returnXML,
                            sizeOfReturnXML) == S_OK) {

                        bDisplayError = false;
                        if (doMagstripeEncode) {
                            bDisplayError = true;
                            sizeOfReturnXML[0] = XPS_Java_SDK.BUFFSIZE;
                            String track1Data = "SMARTCARD MAGNETIC STRIPE ENCODE";
                            String track2Data = "123";
                            String track3Data = "678";
                            if (S_OK == XpsDriverInteropLib.INSTANCE.MagstripeEncode(mPrinterName,
                                    track1Data, track1Data.length(), track2Data, track2Data.length(),
                                    track3Data, track3Data.length(), returnXML, sizeOfReturnXML)) {
                                System.out
                                        .format(
                                                "magstripe encode succeeded.\nTrack 1: %s\nTrack 2: %s\nTrack3: %s\n",
                                                track1Data, track2Data, track3Data);
                                bDisplayError = false;
                            }
                        }
                    }
                } else {
                    // when smartcard personalization fails, cancel the print
                    // job:
                    hResult = XpsDriverInteropLib.INSTANCE.CancelJob(mPrinterName,
                            printerStatusXml.GetPrintJobID(), printerStatusXml.GetErrorCode(), returnXML,
                            sizeOfReturnXML);
                    bDoPrint = false;
                    bJobHasBeenCanceled = true;
                    bDisplayError = (hResult != S_OK);
                }
            }
        }

        if (bDisplayError) {
            printerStatusXml.Parse(returnXML, sizeOfReturnXML);
            System.out.format("Request failed. Reason: %s\n", printerStatusXml.GetErrorMessage());

            System.out.println("Cancelling operation.");
            XpsDriverInteropLib.INSTANCE.SendResponseToPrinter(mPrinterName,
                    printerStatusXml.PRINTERACTION_CANCEL, printerStatusXml.GetPrintJobID(),
                    printerStatusXml.GetErrorCode());
            bJobHasBeenCanceled = true;

        } else if (bDoPrint) {
            // print, no encode again
            JavaPrint javaPrint = new JavaPrint(
                    mPrinterName,
                    isDuplex,
                    isLandscape,
                    false,
                    doMagstripeEncode,
                    true,
                    1);
            javaPrint.PrintDemo();

            Util.WaitUntilJobSpooled(mPrinterName);
        }

        if (bJobStarted && !bJobHasBeenCanceled) {
            System.out.println("Calling EndJob():");
            XpsDriverInteropLib.INSTANCE.EndJob(mPrinterName);
        }

        // (optional) polling Job status
        Util.PollForJobCompletion(mPrinterName, printerStatusXml.GetPrintJobID());

        return (hResult == S_OK);
    }
}
