////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Datacard Corporation.  All Rights Reserved.
////////////////////////////////////////////////////////////////////////////////
package com.sun.jna.examples;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import com.datacard.xpsprinter.sdk.examples.JavaPrint;
import com.datacard.xpsprinter.sdk.examples.Magstripe;
import com.datacard.xpsprinter.sdk.examples.PrinterCounterStatus;
import com.datacard.xpsprinter.sdk.examples.PrinterOptions;
import com.datacard.xpsprinter.sdk.examples.PrinterSuppliesStatus;
import com.datacard.xpsprinter.sdk.examples.SmartCard;
import com.datacard.xpsprinter.sdk.examples.Util;
import com.sun.jna.Library;
import com.sun.jna.Native;
import java.nio.file.FileSystems;
import java.nio.file.Path;

public class XPS_Java_SDK {
    public final static int BUFFSIZE = 10240; // buffer size for all the interop communications:

    // Java interop DLL:
    public final static String INTEROPDLL = "dxp01sdk_IBidiSpl_interop.dll";

    // xps printer interface and functions
    public interface XpsDriverInteropLib extends Library {

        XpsDriverInteropLib INSTANCE = (XpsDriverInteropLib) Native.loadLibrary(
                INTEROPDLL, XpsDriverInteropLib.class);

        int GetSDKVersion(String printerName, byte returnBuffer[],
                int bufferSize[]);

        int GetPrinterOptions2(String printerName, byte returnBuffer[],
                int bufferSize[]);

        int GetPrinterCounterStatus2(String printerName, byte returnBuffer[],
                int bufferSize[]);

        int GetPrinterSuppliesStatus(String printerName, byte returnBuffer[],
                int bufferSize[]);

        int StartJob(String printerName, byte returnXML[], int sizeOfReturnXML[]);

        int MagstripeRead(String printerName, byte dataBuffer[], int bufferSize[]);

        int MagstripeEncode(String printerName, String track1Data,
                int sizeOfTrack1Data, String track2Data, int sizeOfTrack2Data,
                String track3Data, int sizeOfTrack3Data, byte returnXML[],
                int sizeOfReturnXML[]);

        boolean SendResponseToPrinter(String aPrinterName, int command,
                int printerJobID, int errorCode);

        int SmartCardPark(
                String aPrinterName,
                byte returnXML[],
                int sizeOfReturnXML[],
                boolean parkBack);

        int SCardConnect(String aPrinterName, int connectType, int protocol[]);

        int SCardDisConnect(String aPrinterName, int disposition);

        int SCardStatus(
                String aPrinterName,
                int states[],
                int numberStatesReturned[],
                int protocol[],
                byte returnATRBytes[],
                int numATRBytesReturned[]);

        int SCardGetAttrib(String aPrinterName, int attrID,
                byte attrBytesBuffer[], int attrBytesBufferSize[]);

        int SCardTransmit(String aPrinterName, byte sendBytesBuffer[],
                int sendBytesBufferSize, byte receivedBytesBuffer[],
                int receivedBytesBufferSize[]);

        int ResumeJob(String printerName, int printerJobID, int errorCode,
                byte returnXML[], int sizeOfReturnXML[]);

        int CancelJob(String printerName, int printerJobID, int errorCode,
                byte returnXML[], int sizeOfReturnXML[]);

        int EndJob(String aPrinterName);

        int GetJobStatusXML(String aPrinterName, int aPrinterJobID,
                byte returnXML[], int sizeOfReturnXML[]);

        int GetPrinterStatus(String schemaString, String printerName,
                byte returnBuffer[], int bufferSize[]);
    }

    // make sure JNA can find xps printer interface dxp01sdk_IBidiSpl_interop.dll
    static {
        System.setProperty("jna.library.path", ".;" + System.getProperty("os.arch"));
        extractDll(System.getProperty("os.arch"), INTEROPDLL);
    }

    public static void main(String[] args) {
        String printerName = "";
        int i = 0;
        String arg;

        // parse the command line arguments
        while (i < args.length && args[i].startsWith("-")) {
            arg = args[i++];

            if (arg.equalsIgnoreCase("-n")) {
                if (i < args.length) {
                    printerName = args[i++];
                } else {
                    System.err.println("-n requires a printername");
                    break;
                }
            } else if (arg.startsWith("-mr")) {
                System.out.format(
                        "Magstripe Read Demo\n\noptions ---------------------\nprint: %b\nisLandscape: %b\nisDuplex: %b\n",
                        (arg.length() > 3 && arg.contains("p")), arg.contains("l"),
                        arg.contains("d"));
                Magstripe magStripe = new Magstripe(printerName);
                magStripe.Read((arg.length() > 3 && arg.contains("p")),
                        arg.contains("l"), arg.contains("d"));
            } else if (arg.startsWith("-me")) {
                System.out.format(
                        "Magstripe Encode Demo\n\noptions ---------------------\nmagstripe read: %b\nprint: %b\nisLandscape: %b\nisDuplex: %b\nJIS magnetic: %b\n",
                        (arg.length() > 3 && arg.contains("mr")),
                        (arg.length() > 3 && arg.contains("p")), arg.contains("l"),
                        arg.contains("d"), arg.contains("j"));
                Magstripe magStripe = new Magstripe(printerName);
                boolean isJIS = arg.contains("j");
                if (!isJIS) {
                    magStripe.Encode(
                            " !#$%&\'\"()*+,-./0123456789:;<=>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !#$%&\'()*+,",
                            "0123456789:;<=>0123456789:;<=>0123456",
                            "0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=>0123456789:;<=",
                            (arg.length() > 3 && arg.contains("mr")),
                            (arg.length() > 3 && arg.contains("p")),
                            arg.contains("l"), arg.contains("d"));
                } else {
                    // You need make sure your printer has JIS magstripe head and any
                    // necessary setting. The key here is the data should put into track 3
                    // only.
                    String nttData = "!\"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNO";
                    magStripe.Encode("", "", nttData,
                            (arg.length() > 3 && arg.contains("mr")),
                            (arg.length() > 3 && arg.contains("p")),
                            arg.contains("l"), arg.contains("d"));
                }
            } else if (arg.equalsIgnoreCase("-s")) {
                System.out.println("Printer Options Demo");
                PrinterOptions printerOptions = new PrinterOptions(printerName);
                printerOptions.GetPrinterOptions();
            } else if (arg.equalsIgnoreCase("-cc")) {
                System.out.println("Printer Card Counts Demo");
                PrinterCounterStatus printerOptions = new PrinterCounterStatus(
                        printerName);
                printerOptions.GetPrinterCounterStatus();
            } else if (arg.equalsIgnoreCase("-ss")) {
                System.out.println("Printer Supplies Status Demo");
                PrinterSuppliesStatus printerSuppliesStatus = new PrinterSuppliesStatus(
                        printerName);
                printerSuppliesStatus.GetPrinterSuppliesStatus();
            } else if (arg.startsWith("-sc")) {
                final boolean doMagStripeEncode = arg.length() > 3 && arg.contains("me");
                final boolean doPrint = arg.length() > 3 && arg.contains("p");
                final boolean landscape = arg.contains("l");
                final boolean duplex = arg.contains("d");
                final boolean singleWire = arg.contains("sw");
                final boolean parkBack = arg.contains("b");
                final String format = "Smart Card Demo options:"
                        + "\n  magstripe encode: %b"
                        + "\n  print: %b"
                        + "\n  landscape: %b"
                        + "\n  duplex: %b"
                        + "\n  singlewire: %b"
                        + "\n  parkback: %b"
                        + "\n";
                System.out.format(format,
                        doMagStripeEncode,
                        doPrint,
                        landscape,
                        duplex,
                        singleWire,
                        parkBack);

                SmartCard smartCard = new SmartCard(printerName);
                smartCard.DemoSmartCard(
                        doMagStripeEncode,
                        doPrint,
                        landscape,
                        duplex,
                        singleWire,
                        parkBack);
            } else if (arg.startsWith("-p")) {
                String tempArg = arg.substring(2);
                System.out.format(
                        "PrintCard Demo\n\noptions ---------------------\nisLandscape: %b\nisDuplex: %b\nback side encode: %b\nJIS magnetic: %b\ncopy count: %d\n",
                        (tempArg.contains("l")), (tempArg.contains("d")),
                        (tempArg.contains("e")), tempArg.contains("j"),
                        getCopyCount(arg));
                System.out.println();
                JavaPrint javaPrint = new JavaPrint(printerName,
                        tempArg.contains("d"), tempArg.contains("l"),
                        tempArg.contains("e"), tempArg.contains("e") // block magstripe
                        , false, getCopyCount(arg));
                javaPrint.PrintDemo();
            } else if (arg.equalsIgnoreCase("-cAll")) {
                System.out.println("CancelAllJobs demo");
                Util.CancelAllJobs(printerName);
            } else {
                usage();
                System.exit(0);
            }
        }

        if (i < 3) {
            usage();
            System.exit(0);
        }
    }

    public static void usage() {
        System.out.println();
        System.out.println("Demonstrates java functionality of the printer and driver.");
        System.out.println();
        System.out.println("Uses hard-coded data for printing, magnetic stripe, topcoat region and print");
        System.out.println("blocking region.");
        System.out.println("Honors the driver printing preferences when options are not set to override.");
        System.out.println("Default orientation is portrait");
        System.out.println("The [e] option also demonstrates how topcoat and print blocking escapes are ");
        System.out.println("used to protect the region over the backside ISO 3-track magnetic stripe.");
        System.out.println();
        System.out.println("java -jar dxp01Java_SDK.jar -n <printername> <function>[options]");
        System.out.println();
        System.out.println("  -n <printername>. Required. Try -n \"xps card printer\"");
        System.out.println();
        System.out.println("  <function>                        Valid options");
        System.out.println("  -p Print                          [l][d][e][j][number]");
        System.out.println("  -mr Magnetic stripe read          [p][l][d]");
        System.out.println("  -me Magnetic stripe encode        [mr][p][l][d][j]");
        System.out.println("  -sc Park a smart card             [me][p][l][d][sw][b]");
        System.out.println("  -s  Get printer information 2     [no options]");
        System.out.println("  -cc Get printer card counts 2     [no options]");
        System.out.println("  -ss Get printer supplies status   [no options]");
        System.out.println("  -cAll Cancel all printer Jobs     [no options]");
        System.out.println();
        System.out.println("  [Options]");
        System.out.println("  j: JIS magnetic");
        System.out.println("  p: print sample text");
        System.out.println("  l: landscape orientation (both sides)");
        System.out.println("  d: print a two-sided (duplex) card");
        System.out.println("  number: copy count");
        System.out.println("  e: encode back-side ISO 3-track magnetic stripe using escapes");
        System.out.println("  e j: encode back-side JIS track 3 magnetic stripe using magnetic font");
        System.out.println("  mr: read back-side ISO 3-track magnetic stripe using interactive mode");
        System.out.println("  me: encode back-side ISO 3-track magnetic stripe using interactive mode");
        System.out.println("  sw: Singlewire smart card demo");
        System.out.println("  b: use back of card for smartcard chip");
        System.out.println();
        System.out.println("Hardcoded values in sample");
        System.out.println(" -me  function: ");
        System.out.println("   track1: \" !#$%&\'\"()*+,-./0123456789:;<=>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !#$%&\'()*+,");
        System.out.println("   track2: 0123456789:;<=>0123456789:;<=>0123456");
        System.out.println("   track3: 0123456789:;<=>0123456789:;... repeat to max length");
        System.out.println();
        System.out.println("  [e]  option:");
        System.out.println("  track1: ABC 123, track2: 345, track3: 789");
        System.out.println();
        System.out.println("  [me] option:");
        System.out.println("  track1: SMARTCARD MAGNETIC STRIPE ENCODE, track2: 123, track3: 678");
        System.out.println("---------------------------------------------------------");
        System.out.println("Examples: (Assume printer name is \"XPS Card Printer\".)");
        System.out.println();
        System.out.println("print 2-sided landscape 2 copies.");
        System.out.println("  java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -pld2");
        System.out.println();
        System.out.println("print 2-sided portrait and encode magnetic stripe using escapes.");
        System.out.println("java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -pde");
        System.out.println();
        System.out.println("Interactive magnetic stripe encode, followed by read, followed by printing");
        System.out.println("a two-sided landscape card.");
        System.out.println("  java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -memrpld");
        System.out.println();
        System.out.println("Interactive smart card park, followed by resume, followed by interactive");
        System.out.println("magnetic stripe encode, followed by printing a two-sided landscape card.");
        System.out.println("  java -jar dxp01Java_SDK.jar -n \"XPS Card Printer\" -scmepld");
        System.out.println();
    }

    private static int getCopyCount(String line) {
        int copyCount = 1;
        try {
            copyCount = Integer.parseInt(line.substring(line.length() - 1));
        } catch (NumberFormatException e) {
            copyCount = 1;
        }
        return copyCount;
    }

    private static void DisplayInteropDllName(String dllName) {
        Path basePath = FileSystems.getDefault().getPath(dllName);
        Path absolutePath = basePath.toAbsolutePath();
        System.out.format(" basePath: %s\n absolutePath: %s",
                basePath,
                absolutePath);
        System.out.println();
    }

    private static void extractDll(String path, String name) {
        try {
            String resourcePath = path + "/" + name;
            URL url = ClassLoader.getSystemResource(resourcePath);
            if (url == null) {
                System.out.println("unable to locate interop DLL.");
                DisplayInteropDllName(resourcePath);
                System.exit(1);
            }

            InputStream in = ClassLoader.getSystemResourceAsStream(resourcePath);
            OutputStream os = new FileOutputStream(name);

            byte[] buffer = new byte[BUFFSIZE];
            int length;
            while ((length = in.read(buffer)) > 0) {
                os.write(buffer, 0, length);
            }
            os.close();
            in.close();
        } catch (Exception e) {
            System.out.format("extractDll() error: %s. Reason: %s", INTEROPDLL, e.getMessage());
            System.exit(0);
        }
    }
}
