''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.Drawing
Imports System.Drawing.Printing
Imports System.IO
Imports System.Printing
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Threading
Imports System.Xml

Namespace dxp01sdk

    ''' <summary>
    ''' we need the printerJobID and and printer error code to properly
    ''' issue a CancelJob() call.
    ''' </summary>
    Public Class BidiException
        Inherits Exception
        Public Sub New(message As String, printerJobID As Integer, errorCode As Integer)
            MyBase.New(message)
            Me.m_errorCode = errorCode
            Me.m_printerJobID = printerJobID
        End Sub

        Public ReadOnly Property PrinterJobID() As Integer
            Get
                Return m_printerJobID
            End Get
        End Property
        Public ReadOnly Property ErrorCode() As Integer
            Get
                Return m_errorCode
            End Get
        End Property

        Private m_printerJobID As Integer
        Private m_errorCode As Integer
    End Class

    Public Class PrinterStatusValues
        Public _clientID As String
        Public _dataFromPrinter As String
        Public _errorCode As Integer
        Public _errorSeverity As Integer
        Public _errorString As String
        Public _printerJobID As Integer
        Public _windowsJobID As Integer
    End Class

    ''' <summary>
    ''' values parsed from a GetPrinterData(strings.PRINTER_OPTIONS2) call.
    ''' </summary>
    Public Class PrinterOptionsValues
        Public _colorPrintResolution As String
        Public _connectionPortType As String
        Public _connectionProtocol As String
        Public _embosserVersion As String
        Public _lockState As String
        Public _moduleEmbosser As String
        Public _monochromePrintResolution As String
        Public _optionDuplex As String
        Public _optionInputhopper As String
        Public _optionLocks As String
        Public _optionMagstripe As String
        Public _optionSmartcard As String
        Public _printerAddress As String
        Public _printerMessageNumber As Integer
        Public _printerModel As String
        Public _printerSerialNumber As String
        Public _printerStatus As String
        Public _printerVersion As String
        Public _printHead As String
        Public _optionPrinterBarcodeReader As String
        Public _optionRewrite As String
        Public _topcoatPrintResolution As String
        Public _laminatorFirmwareVersion As String
        Public _laminator As String
        Public _laminatorImpresser As String
        Public _laminatorScanner As String
    End Class

    Public Class PrinterCounterStatus
        Public _cardsPickedSinceCleaningCard As Integer
        Public _cleaningCardsRun As Integer
        Public _currentCompleted As Integer
        Public _currentLost As Integer
        Public _currentPicked As Integer
        Public _currentPickedExceptionSlot As Integer
        Public _currentPickedInputHopper1 As Integer
        Public _currentPickedInputHopper2 As Integer
        Public _currentPickedInputHopper3 As Integer
        Public _currentPickedInputHopper4 As Integer
        Public _currentPickedInputHopper5 As Integer
        Public _currentPickedInputHopper6 As Integer
        Public _currentRejected As Integer
        Public _printerStatus As String
        Public _totalCompleted As Integer
        Public _totalLost As Integer
        Public _totalPicked As Integer
        Public _totalPickedExceptionSlot As Integer
        Public _totalPickedInputHopper1 As Integer
        Public _totalPickedInputHopper2 As Integer
        Public _totalPickedInputHopper3 As Integer
        Public _totalPickedInputHopper4 As Integer
        Public _totalPickedInputHopper5 As Integer
        Public _totalPickedInputHopper6 As Integer
        Public _totalRejected As Integer
    End Class

    Public Class SuppliesValues
        Public _printerStatus As String
        Public _printRibbonType As String
        Public _ribbonRemaining As Short
        Public _printRibbonSerialNumber As String
        Public _printRibbonLotCode As String
        Public _printRibbonPartNumber As Integer
        Public _printRibbonRegionCode As Integer

        Public _indentRibbon As String
        Public _indentRibbonRemaining As Short
        Public _indentRibbonSerialNumber As String
        Public _indentRibbonLotCode As String
        Public _indentRibbonPartNumber As Integer

        Public _topperRibbonType As String
        Public _topperRibbonRemaining As Short
        Public _topperRibbonSerialNumber As String
        Public _topperRibbonLotCode As String
        Public _topperRibbonPartNumber As Integer

        Public _laminatorL1SupplyCode As Long
        Public _laminatorL1PercentRemaining As Short
        Public _laminatorL1SerialNumber As String
        Public _laminatorL1LotCode As String
        Public _laminatorL1PartNumber As Integer

        Public _laminatorL2SupplyCode As Long
        Public _laminatorL2PercentRemaining As Short
        Public _laminatorL2SerialNumber As String
        Public _laminatorL2LotCode As String
        Public _laminatorL2PartNumber As Integer
    End Class

    Public Class JobStatusValues
        Public _clientID As String
        Public _jobRestartCount As Integer
        Public _jobState As String
        Public _printerJobID As Integer
        Public _windowsJobID As Integer
    End Class

    Public Class SmartcardResponseValues
        Public _protocol As String
        Public _states As String()
        Public _status As String
        Public _base64string As String
        Public _bytesFromBase64String As Byte()
    End Class

    Class Util
        Private Enum PrintSpoolerStatus
            Other = 1
            Unknown
            Idle
            Printing
            WarmingUp
            StoppedPrinting
            Offline
        End Enum

        Public Shared Function StringFromXml(doc As XmlDocument, xmlElementName As String) As String
            Dim node As XmlNode = doc.SelectSingleNode(xmlElementName)
            If node IsNot Nothing Then
                Return node.InnerText
            End If
            Return String.Empty
        End Function

        Public Shared Function IntFromXml(doc As XmlDocument, xmlElementName As String) As Integer
            Dim node As XmlNode = doc.SelectSingleNode(xmlElementName)
            Dim val As Integer = 0
            If node IsNot Nothing Then
                Int32.TryParse(node.InnerText, val)
            End If
            Return val
        End Function

        ''' <summary>
        ''' ParsePrinterOptionsXML()
        '''
        ''' we have PrinterOptions2 xml from the printer. Parse the xml and populate
        ''' a new PrinterOptionsValues instance with the data items.
        '''
        '''     <?xml version="1.0"?>
        '''     <!--Printer options2 xml file.-->
        '''     <PrinterInfo2>
        '''         <ColorPrintResolution>300x300 | 300x600</ColorPrintResolution>
        '''         <ConnectionPortType>Network</ConnectionPortType>
        '''         <ConnectionProtocol>Version2Secure</ConnectionProtocol>
        '''         <EmbosserVersion>E1.1.25-0</EmbosserVersion>
        '''         <LockState>Locked</LockState>
        '''         <ModuleEmbosser>Installed</ModuleEmbosser>
        '''         <MonochromePrintResolution>300x300 | 300x600 | 300x1200</MonochromePrintResolution>
        '''         <OptionDuplex>Auto</OptionDuplex>
        '''         <OptionInputhopper>MultiHopper6</OptionInputhopper>
        '''         <OptionLocks>Installed</OptionLocks>
        '''         <OptionMagstripe>ISO</OptionMagstripe>
        '''         <OptionSmartcard>Single wire</OptionSmartcard>
        '''         <PrinterAddress>172.16.2.211</PrinterAddress>
        '''         <PrinterMessageNumber>0</PrinterMessageNumber>
        '''         <PrinterModel>CD870</PrinterModel>
        '''         <PrinterSerialNumber>E00105</PrinterSerialNumber>
        '''         <PrinterStatus>Ready</PrinterStatus>
        '''         <PrinterVersion>D3.12.4-0</PrinterVersion>
        '''         <PrintHead>Installed</PrintHead>
        '''         <TopcoatPrintResolution>300x300</TopcoatPrintResolution>
        '''         <Laminator>L1,L2</Laminator>
        '''         <LaminatorImpresser>None</LaminatorImpresser>
        '''         <LaminatorScanner>None</LaminatorScanner>
        '''     </PrinterInfo2>
        '''
        ''' </summary>
        ''' <param name="printerOptionsXML"></param>
        ''' <returns>a populated PrinterOptionsValues struct</returns>
        Public Shared Function ParsePrinterOptionsXML(printerOptionsXML As String) As PrinterOptionsValues
            Dim doc As New XmlDocument()
            doc.LoadXml(printerOptionsXML)

            Dim printerOptionsValues As New PrinterOptionsValues()

            printerOptionsValues._colorPrintResolution = StringFromXml(doc, "PrinterInfo2/ColorPrintResolution")
            printerOptionsValues._connectionPortType = StringFromXml(doc, "PrinterInfo2/ConnectionPortType")
            printerOptionsValues._connectionProtocol = StringFromXml(doc, "PrinterInfo2/ConnectionProtocol")
            printerOptionsValues._embosserVersion = StringFromXml(doc, "PrinterInfo2/EmbosserVersion")
            printerOptionsValues._lockState = StringFromXml(doc, "PrinterInfo2/LockState")
            printerOptionsValues._moduleEmbosser = StringFromXml(doc, "PrinterInfo2/ModuleEmbosser")
            printerOptionsValues._monochromePrintResolution = StringFromXml(doc, "PrinterInfo2/MonochromePrintResolution")
            printerOptionsValues._optionDuplex = StringFromXml(doc, "PrinterInfo2/OptionDuplex")
            printerOptionsValues._optionInputhopper = StringFromXml(doc, "PrinterInfo2/OptionInputhopper")
            printerOptionsValues._optionLocks = StringFromXml(doc, "PrinterInfo2/OptionLocks")
            printerOptionsValues._optionMagstripe = StringFromXml(doc, "PrinterInfo2/OptionMagstripe")
            printerOptionsValues._optionSmartcard = StringFromXml(doc, "PrinterInfo2/OptionSmartcard")
            printerOptionsValues._printerAddress = StringFromXml(doc, "PrinterInfo2/PrinterAddress")
            printerOptionsValues._printerMessageNumber = IntFromXml(doc, "PrinterInfo2/PrinterMessageNumber")
            printerOptionsValues._printerModel = StringFromXml(doc, "PrinterInfo2/PrinterModel")
            printerOptionsValues._printerSerialNumber = StringFromXml(doc, "PrinterInfo2/PrinterSerialNumber")
            printerOptionsValues._printerStatus = StringFromXml(doc, "PrinterInfo2/PrinterStatus")
            printerOptionsValues._printerVersion = StringFromXml(doc, "PrinterInfo2/PrinterVersion")
            printerOptionsValues._printHead = StringFromXml(doc, "PrinterInfo2/PrintHead")
            printerOptionsValues._optionRewrite = StringFromXml(doc, "PrinterInfo2/OptionRewritable")
            printerOptionsValues._optionPrinterBarcodeReader = StringFromXml(doc, "PrinterInfo2/OptionPrinterBarcodeReader")
            printerOptionsValues._topcoatPrintResolution = StringFromXml(doc, "PrinterInfo2/TopcoatPrintResolution")

            printerOptionsValues._laminatorFirmwareVersion = StringFromXml(doc, "PrinterInfo2/LaminatorFirmwareVersion")
            printerOptionsValues._laminator = StringFromXml(doc, "PrinterInfo2/Laminator")
            printerOptionsValues._laminatorImpresser = StringFromXml(doc, "PrinterInfo2/LaminatorImpresser")
            printerOptionsValues._laminatorScanner = StringFromXml(doc, "PrinterInfo2/LaminatorScanner")

            Return printerOptionsValues
        End Function

        ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        ' ParsePrinterCounterStatusXML()
        '
        ' the 'printer card counts' xml from the printer looks something like this:
        '
        '    <?xml version="1.0"?>
        '    <!--Printer counter2 xml file.-->
        '    <CounterStatus2>
        '       <CardsPickedSinceCleaningCard>474</CardsPickedSinceCleaningCard>
        '       <CleaningCardsRun>3</CleaningCardsRun>
        '       <CurrentCompleted>260</CurrentCompleted>
        '       <CurrentLost>0</CurrentLost>
        '       <CurrentPicked>474</CurrentPicked>
        '       <CurrentPickedExceptionSlot>2</CurrentPickedExceptionSlot>
        '       <CurrentPickedInputHopper1>242</CurrentPickedInputHopper1>
        '       <CurrentPickedInputHopper2>1</CurrentPickedInputHopper2>
        '       <CurrentPickedInputHopper3>33</CurrentPickedInputHopper3>
        '       <CurrentPickedInputHopper4>14</CurrentPickedInputHopper4>
        '       <CurrentPickedInputHopper5>3</CurrentPickedInputHopper5>
        '       <CurrentPickedInputHopper6>1</CurrentPickedInputHopper6>
        '       <CurrentRejected>36</CurrentRejected>
        '       <PrinterStatus>Ready</PrinterStatus>
        '       <TotalCompleted>260</TotalCompleted>
        '       <TotalLost>0</TotalLost>
        '       <TotalPicked>474</TotalPicked>
        '       <TotalPickedExceptionSlot>2</TotalPickedExceptionSlot>
        '       <TotalPickedInputHopper1>242</TotalPickedInputHopper1>
        '       <TotalPickedInputHopper2>1</TotalPickedInputHopper2>
        '       <TotalPickedInputHopper3>33</TotalPickedInputHopper3>
        '       <TotalPickedInputHopper4>14</TotalPickedInputHopper4>
        '       <TotalPickedInputHopper5>3</TotalPickedInputHopper5>
        '       <TotalPickedInputHopper6>1</TotalPickedInputHopper6>
        '       <TotalRejected>36</TotalRejected>
        '    </CounterStatus2>
        '
        ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Public Shared Function ParsePrinterCounterStatusXML(printercardCountXML As String) As PrinterCounterStatus
            Dim doc As New XmlDocument()
            doc.LoadXml(printercardCountXML)

            Dim printerCardCountValues As New PrinterCounterStatus()

            printerCardCountValues._cardsPickedSinceCleaningCard = IntFromXml(doc, "CounterStatus2/CardsPickedSinceCleaningCard")
            printerCardCountValues._cleaningCardsRun = IntFromXml(doc, "CounterStatus2/CleaningCardsRun")
            printerCardCountValues._currentCompleted = IntFromXml(doc, "CounterStatus2/CurrentCompleted")
            printerCardCountValues._currentLost = IntFromXml(doc, "CounterStatus2/CurrentLost")
            printerCardCountValues._currentPicked = IntFromXml(doc, "CounterStatus2/CurrentPicked")
            printerCardCountValues._currentPickedExceptionSlot = IntFromXml(doc, "CounterStatus2/CurrentPickedExceptionSlot")
            printerCardCountValues._currentPickedInputHopper1 = IntFromXml(doc, "CounterStatus2/CurrentPickedInputHopper1")
            printerCardCountValues._currentPickedInputHopper2 = IntFromXml(doc, "CounterStatus2/CurrentPickedInputHopper2")
            printerCardCountValues._currentPickedInputHopper3 = IntFromXml(doc, "CounterStatus2/CurrentPickedInputHopper3")
            printerCardCountValues._currentPickedInputHopper4 = IntFromXml(doc, "CounterStatus2/CurrentPickedInputHopper4")
            printerCardCountValues._currentPickedInputHopper5 = IntFromXml(doc, "CounterStatus2/CurrentPickedInputHopper5")
            printerCardCountValues._currentPickedInputHopper6 = IntFromXml(doc, "CounterStatus2/CurrentPickedInputHopper6")
            printerCardCountValues._currentRejected = IntFromXml(doc, "CounterStatus2/CurrentRejected")
            printerCardCountValues._printerStatus = StringFromXml(doc, "CounterStatus2/PrinterStatus")
            printerCardCountValues._totalCompleted = IntFromXml(doc, "CounterStatus2/TotalCompleted")
            printerCardCountValues._totalLost = IntFromXml(doc, "CounterStatus2/TotalLost")
            printerCardCountValues._totalPicked = IntFromXml(doc, "CounterStatus2/TotalPicked")
            printerCardCountValues._totalPickedExceptionSlot = IntFromXml(doc, "CounterStatus2/TotalPickedExceptionSlot")
            printerCardCountValues._totalPickedInputHopper1 = IntFromXml(doc, "CounterStatus2/TotalPickedInputHopper1")
            printerCardCountValues._totalPickedInputHopper2 = IntFromXml(doc, "CounterStatus2/TotalPickedInputHopper2")
            printerCardCountValues._totalPickedInputHopper3 = IntFromXml(doc, "CounterStatus2/TotalPickedInputHopper3")
            printerCardCountValues._totalPickedInputHopper4 = IntFromXml(doc, "CounterStatus2/TotalPickedInputHopper4")
            printerCardCountValues._totalPickedInputHopper5 = IntFromXml(doc, "CounterStatus2/TotalPickedInputHopper5")
            printerCardCountValues._totalPickedInputHopper6 = IntFromXml(doc, "CounterStatus2/TotalPickedInputHopper6")
            printerCardCountValues._totalRejected = IntFromXml(doc, "CounterStatus2/TotalRejected")

            Return printerCardCountValues
        End Function

        Public Shared Function ParseSuppliesXML(suppliesXML As String) As SuppliesValues

            ' sample supplies xml from printer:
            '
            ' <?xml version="1.0"?>
            ' <!--Printer Supplies3 xml file.-->
            ' <PrinterSupplies3>
            '   <PrinterStatus>Ready</PrinterStatus>
            '   <Printer>
            '     <PrintRibbon>Installed</PrintRibbon>
            '     <PrintRibbonType>Monochrome</PrintRibbonType>
            '     <RibbonLotCode>16MAR12  </RibbonLotCode>
            '     <RibbonPartNumber>533000052</RibbonPartNumber>
            '     <RibbonRemaining>43</RibbonRemaining>
            '     <RibbonSerialNumber>E0055000008E9956</RibbonSerialNumber>
            '   </Printer>
            '   <Embosser>
            '     <IndentRibbon>Installed</IndentRibbon>
            '     <IndentRibbonLotCode/>
            '     <IndentRibbonPartNumber/>
            '     <IndentRibbonRemaining>50</IndentRibbonRemaining>
            '     <IndentRibbonSerialNumber/>
            '     <IndentRibbonType>67174401</IndentRibbonType>
            '     <TopperRibbon>Installed</TopperRibbon>
            '     <TopperRibbonLotCode>29Oct11  </TopperRibbonLotCode>
            '     <TopperRibbonPartNumber>504139013</TopperRibbonPartNumber>
            '     <TopperRibbonRemaining>25</TopperRibbonRemaining>
            '     <TopperRibbonSerialNumber>25A18E00005005E0</TopperRibbonSerialNumber>
            '     <TopperRibbonType>Silver</TopperRibbonType>
            '   </Embosser>
            '   <Laminator>
            '     <L1Laminate>None</L1Laminate>
            '     <L1LaminateLotCode/>
            '     <L1LaminatePartNumber/>
            '     <L1LaminateRemaining/>
            '     <L1LaminateSerialNumber/>
            '     <L1LaminateType/>
            '     <L2Laminate>None</L2Laminate>
            '     <L2LaminateLotCode/>
            '     <L2LaminatePartNumber/>
            '     <L2LaminateRemaining/>
            '     <L2LaminateSerialNumber/>
            '     <L2LaminateType/>
            '   </Laminator>
            ' </PrinterSupplies3>

            Dim doc As New XmlDocument()
            doc.LoadXml(suppliesXML)

            Dim suppliesValues As New SuppliesValues()

            suppliesValues._indentRibbon = StringFromXml(doc, "PrinterSupplies3/Embosser/IndentRibbon")
            suppliesValues._indentRibbonLotCode = StringFromXml(doc, "PrinterSupplies3/Embosser/IndentRibbonLotCode")
            suppliesValues._indentRibbonPartNumber = CInt(IntFromXml(doc, "PrinterSupplies3/Embosser/IndentRibbonPartNumber"))
            suppliesValues._indentRibbonRemaining = CShort(IntFromXml(doc, "PrinterSupplies3/Embosser/IndentRibbonRemaining"))
            suppliesValues._indentRibbonSerialNumber = StringFromXml(doc, "PrinterSupplies3/Embosser/IndentRibbonSerialNumber")
            suppliesValues._laminatorL1LotCode = StringFromXml(doc, "PrinterSupplies3/Laminator/L1LaminateLotCode")
            suppliesValues._laminatorL1PartNumber = CInt(IntFromXml(doc, "PrinterSupplies3/Laminator/L1LaminatePartNumber"))
            suppliesValues._laminatorL1PercentRemaining = CShort(IntFromXml(doc, "PrinterSupplies3/Laminator/L1LaminateRemaining"))
            suppliesValues._laminatorL1SerialNumber = StringFromXml(doc, "PrinterSupplies3/Laminator/L1LaminateSerialNumber")
            suppliesValues._laminatorL1SupplyCode = CLng(IntFromXml(doc, "PrinterSupplies3/Laminator/L1LaminateType"))
            suppliesValues._laminatorL2LotCode = StringFromXml(doc, "PrinterSupplies3/Laminator/L2LaminateLotCode")
            suppliesValues._laminatorL2PartNumber = CInt(IntFromXml(doc, "PrinterSupplies3/Laminator/L2LaminatePartNumber"))
            suppliesValues._laminatorL2PercentRemaining = CShort(IntFromXml(doc, "PrinterSupplies3/Laminator/L2LaminateRemaining"))
            suppliesValues._laminatorL2SerialNumber = StringFromXml(doc, "PrinterSupplies3/Laminator/L2LaminateSerialNumber")
            suppliesValues._laminatorL2SupplyCode = CLng(IntFromXml(doc, "PrinterSupplies3/Laminator/L2LaminateType"))
            suppliesValues._printerStatus = StringFromXml(doc, "PrinterSupplies3/PrinterStatus")
            suppliesValues._printRibbonLotCode = StringFromXml(doc, "PrinterSupplies3/Printer/RibbonLotCode")
            suppliesValues._printRibbonPartNumber = CInt(IntFromXml(doc, "PrinterSupplies3/Printer/RibbonPartNumber"))
            suppliesValues._printRibbonRegionCode = CInt(IntFromXml(doc, "PrinterSupplies3/Printer/RibbonRegionCode"))
            suppliesValues._printRibbonSerialNumber = StringFromXml(doc, "PrinterSupplies3/Printer/RibbonSerialNumber")
            suppliesValues._printRibbonType = StringFromXml(doc, "PrinterSupplies3/Printer/PrintRibbonType")
            suppliesValues._ribbonRemaining = CShort(IntFromXml(doc, "PrinterSupplies3/Printer/RibbonRemaining"))
            suppliesValues._topperRibbonLotCode = StringFromXml(doc, "PrinterSupplies3/Embosser/TopperRibbonLotCode")
            suppliesValues._topperRibbonPartNumber = CInt(IntFromXml(doc, "PrinterSupplies3/Embosser/TopperRibbonPartNumber"))
            suppliesValues._topperRibbonRemaining = CShort(IntFromXml(doc, "PrinterSupplies3/Embosser/TopperRibbonRemaining"))
            suppliesValues._topperRibbonSerialNumber = StringFromXml(doc, "PrinterSupplies3/Embosser/TopperRibbonSerialNumber")
            suppliesValues._topperRibbonType = StringFromXml(doc, "PrinterSupplies3/Embosser/TopperRibbonType")

            Return suppliesValues
        End Function

        ''' <summary>
        ''' populate values from this xml returned from the printer:
        ''' <?xml version="1.0"?>
        ''' <SDKVersion>2.1.217.0</SDKVersion>
        ''' </summary>
        Public Shared Function ParseDriverVersionXML(driverVersionXML As String) As String
            Dim driverVersion As String = String.Empty
            Dim doc As New XmlDocument()
            doc.LoadXml(driverVersionXML)
            Try
                Dim node As XmlNode = doc.SelectSingleNode("SDKVersion")
                driverVersion = node.InnerText
                ' note: driver version not returned in previous drivers.
            Catch e As Exception
            End Try
            Return driverVersion
        End Function

        ''' <summary>
        ''' sample xml printer status xml:
        '''
        '''     <!--Printer status xml file.-->
        '''     <PrinterStatus>
        '''         <ClientID>fellmadwin7x64_{8CB9F232-7136-42BC-B0FA-EF7D734EAEAD}</ClientID>
        '''         <DataFromPrinter><![CDATA[  ]]></DataFromPrinter>
        '''         <ErrorCode>0</ErrorCode>
        '''         <ErrorSeverity>0</ErrorSeverity>
        '''         <ErrorString></ErrorString>
        '''         <PrinterJobID>792</PrinterJobID>
        '''         <WindowsJobID>0</WindowsJobID>
        '''     </PrinterStatus>
        '''
        ''' </summary>
        Public Shared Function ParsePrinterStatusXML(printerStatusXML As String) As PrinterStatusValues
            Dim doc As New XmlDocument()
            doc.LoadXml(printerStatusXML)

            Dim printerStatusValues As New PrinterStatusValues()

            printerStatusValues._printerJobID = IntFromXml(doc, "PrinterStatus/PrinterJobID")
            printerStatusValues._clientID = StringFromXml(doc, "PrinterStatus/ClientID")
            printerStatusValues._dataFromPrinter = StringFromXml(doc, "PrinterStatus/DataFromPrinter")
            printerStatusValues._errorCode = IntFromXml(doc, "PrinterStatus/ErrorCode")
            printerStatusValues._errorSeverity = IntFromXml(doc, "PrinterStatus/ErrorSeverity")
            printerStatusValues._errorString = StringFromXml(doc, "PrinterStatus/ErrorString")
            printerStatusValues._windowsJobID = IntFromXml(doc, "PrinterStatus/WindowsJobID")

            Return printerStatusValues
        End Function

        Public Shared Function ParseJobStatusXML(jobStatusValues As String) As JobStatusValues
            Dim doc As New XmlDocument()
            doc.LoadXml(jobStatusValues)

            Dim jobStatusXML As New JobStatusValues()

            jobStatusXML._clientID = StringFromXml(doc, "JobStatus/ClientID")
            jobStatusXML._jobRestartCount = IntFromXml(doc, "JobStatus/JobRestartCount")
            jobStatusXML._jobState = StringFromXml(doc, "JobStatus/JobState")
            jobStatusXML._printerJobID = IntFromXml(doc, "JobStatus/PrinterJobID")
            jobStatusXML._windowsJobID = IntFromXml(doc, "JobStatus/WindowsJobID")

            Return jobStatusXML
        End Function

        ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        ' ParseMagstripeStrings()
        '
        ' we're given a Printer Status XML fragment that has a CDATA section like this:
        '
        '   <DataFromPrinter>
        '      <![CDATA[<?xml version="1.0" encoding="UTF-8"?>
        '      <magstripe xmlns:SOAP-ENV="http://www.w3.org/2003/05/soap-envelope"
        '         xmlns:SOAP-ENC="http://www.w3.org/2003/05/soap-encoding"
        '         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        '         xmlns:xsd="http://www.w3.org/2001/XMLSchema"
        '         xmlns:DPCLMagStripe="urn:dpcl:magstripe:2010-01-19"
        '         xsi:type="DPCLMagStripe:MagStripe"
        '         SOAP-ENV:encodingStyle="http://www.w3.org/2003/05/soap-encoding">
        '         <track number="1">
        '            <base64Data>R1JPVU5E</base64Data>
        '         </track>
        '         <track number="2">
        '            <base64Data>MTIzNA==</base64Data>
        '         </track>
        '         <track number="3">
        '            <base64Data>MzIx</base64Data>
        '         </track>
        '      </magstripe>]]>
        '   </DataFromPrinter>
        '
        ' return three strings. the string data is still Base64 encoded.
        ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Public Shared Sub ParseMagstripeStrings(printerStatusXML As String, ByRef track1 As String, ByRef track2 As String, ByRef track3 As String, jisRequest As Boolean)
            Dim doc As New XmlDocument()
            doc.LoadXml(printerStatusXML)

            Dim node As XmlNode
            node = doc.SelectSingleNode("PrinterStatus/DataFromPrinter/text()")
            Dim cdataText As String = node.InnerText

            If cdataText.Length > 10 Then
                ' Make sure there is valid data before going further.
                Dim magstripeDoc As New XmlDocument()
                magstripeDoc.LoadXml(cdataText)

                Dim nodelist As XmlNodeList = magstripeDoc.SelectNodes("magstripe/track")
                For Each track As XmlNode In nodelist
                    Dim attrColl As XmlAttributeCollection = track.Attributes
                    Dim attr As XmlAttribute = DirectCast(attrColl.GetNamedItem("number"), XmlAttribute)
                    Dim trackNumber As Integer = Convert.ToInt32(attr.Value)
                    If jisRequest AndAlso trackNumber <> 3 Then
                        ' only track 3 is valid for JIS request
                        Continue For
                    End If
                    Select Case trackNumber

                        Case 1
                            track1 = track.InnerText
                            Exit Select

                        Case 2
                            track2 = track.InnerText
                            Exit Select

                        Case 3
                            track3 = track.InnerText
                            Exit Select
                    End Select
                Next
            End If
        End Sub

        ''' <summary>
        ''' repeatedly get the printer status until with printer status contains
        ''' a valid windows jobID.
        ''' </summary>
        ''' <returns>zero if any error detected, or the windows jobID</returns>
        Public Shared Function WaitForWindowsJobID(bidiSpl As BidiSplWrap, printerName As String) As Integer

            Const sleepMilliSeconds As Integer = 1000

            Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)
            Dim printerStatus As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)

            Do

                If 0 <> printerStatus._windowsJobID Then
                    Console.WriteLine(String.Format("returning windowsJobID {0}.", printerStatus._windowsJobID))
                    Return printerStatus._windowsJobID
                End If

                Console.WriteLine(String.Format("waiting {0} millseconds for windows jobID", sleepMilliSeconds))
                Thread.Sleep(sleepMilliSeconds)

                printerStatusXML = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)

                printerStatus = Util.ParsePrinterStatusXML(printerStatusXML)
            Loop While 0 = printerStatus._errorCode

            Console.WriteLine("printer error:")
            Console.WriteLine(printerStatusXML)

            Return 0
        End Function

        Private Shared Sub OnPrintPage(sender As Object, pageEventArgs As PrintPageEventArgs)
            Using font As New Font("Arial", 8)
                Using brush As New SolidBrush(Color.Black)
                    Using pen As New Pen(Color.DarkBlue)
                        pageEventArgs.Graphics.DrawString("Sample Text", font, brush, 50, 50)
                        pageEventArgs.Graphics.DrawRectangle(pen, 50, 70, 20, 25)
                    End Using
                End Using
            End Using
        End Sub

        Public Shared Sub PrintTextAndGraphics(bidiSpl As BidiSplWrap, printerName As String)
            Dim printDocument As New PrintDocument()
            printDocument.PrintController = New StandardPrintController()
            printDocument.PrinterSettings.PrinterName = printerName
            printDocument.DocumentName = Path.GetFileName(Environment.GetCommandLineArgs()(0)) & " printing"
            AddHandler printDocument.PrintPage, New PrintPageEventHandler(AddressOf OnPrintPage)
            printDocument.Print()

            ' we wait here - to ensure that the job is truly in the windows
            ' spooler:
            Dim windowsJobID As Integer = WaitForWindowsJobID(bidiSpl, printerName)
        End Sub

        Public Shared Sub DisplayPrintTicket(printTicket As PrintTicket)
            Dim memStream As MemoryStream = printTicket.GetXmlStream()
            Dim xml As String = Encoding.ASCII.GetString(memStream.GetBuffer(), 0, CInt(memStream.Length))
            Console.WriteLine(xml)
        End Sub

        ''' <summary>
        ''' incoming xml looks like this:
        '''
        '''  <?xml version="1.0" ?>
        '''  <!-- Printer status xml file.-->
        '''  <PrinterStatus>
        '''    <ClientID>myMachineName_{8CB9F232-7136-42BC-B0FA-EF7D734EAEAD}</ClientID>
        '''    <WindowsJobID>0</WindowsJobID>
        '''    <PrinterJobID>8446</PrinterJobID>
        '''    <ErrorCode>0</ErrorCode>
        '''    <ErrorSeverity>0</ErrorSeverity>
        '''    <ErrorString />
        '''    <DataFromPrinter>
        '''      <![CDATA[ <?xml version="1.0"?>
        '''      <!--smartcard response xml-->
        '''      <SmartcardResponse>
        '''      <Protocol>SCARD_PROTOCOL_RAW</Protocol>
        '''      <State>  </State>
        '''      <Status>SCARD_S_SUCCESS</Status>
        '''      <Base64Data> </Base64Data>
        '''      </SmartcardResponse>]]>
        '''    </DataFromPrinter>
        '''  </PrinterStatus>
        '''
        ''' we retrieve the single-wire smartcard response data from the CDATA section.
        '''
        ''' </summary>
        ''' <param name="printerStatusXml"></param>
        ''' <returns>a populated SmartcardResponseValues object populated from the xml markup.</returns>
        Public Shared Function ParseSmartcardResponseXML(printerStatusXml As String) As SmartcardResponseValues
            Dim smartcardResponseValues As SmartcardResponseValues = New SmartcardResponseValues()

            Dim doc As New XmlDocument()
            doc.LoadXml(printerStatusXml)

            Dim node As XmlNode

            node = doc.SelectSingleNode("PrinterStatus/DataFromPrinter/text()")
            Dim cdataText As String = node.InnerText

            Dim smartcardResponseDoc As New XmlDocument()
            smartcardResponseDoc.LoadXml(cdataText)

            node = smartcardResponseDoc.SelectSingleNode("SmartcardResponse/Protocol")
            smartcardResponseValues._protocol = node.InnerText.Trim()

            node = smartcardResponseDoc.SelectSingleNode("SmartcardResponse/State")
            Dim raw_statestring As String = node.InnerText.Trim()
            Dim statesList As List(Of String) = New List(Of String)()
            For Each stateString As String In raw_statestring.Split(New Char() {"|"c, " "c})
                If Not String.IsNullOrWhiteSpace(stateString) Then
                    statesList.Add(stateString)
                End If
            Next
            smartcardResponseValues._states = statesList.ToArray()

            node = smartcardResponseDoc.SelectSingleNode("SmartcardResponse/Status")
            smartcardResponseValues._status = node.InnerText.Trim()

            node = smartcardResponseDoc.SelectSingleNode("SmartcardResponse/Base64Data")
            smartcardResponseValues._base64string = node.InnerText.Trim()

            If String.IsNullOrEmpty(smartcardResponseValues._base64string) Then
                smartcardResponseValues._bytesFromBase64String = New Byte(-1) {}
            Else
                smartcardResponseValues._bytesFromBase64String = Convert.FromBase64String(smartcardResponseValues._base64string)
            End If

            Return smartcardResponseValues
        End Function

        <DllImport("kernel32.dll")>
        Private Shared Function FormatMessage(
            dwFlags As UInteger,
            lpSource As IntPtr,
            dwMessageId As UInteger,
            dwLanguageId As UInteger,
            <Out> lpBuffer As StringBuilder,
            nSize As UInteger,
            Arguments As IntPtr) As UInteger
        End Function

        ''' <summary>
        ''' get a string result...like the Win32 FormatMessage() function.
        ''' </summary>
        ''' <param name="errno">a numeric win32 error code</param>
        ''' <returns>a localized string corresponding to the given error code</returns>
        Public Shared Function Win32ErrorString(win32error As UInteger) As String
            Const FORMAT_MESSAGE_IGNORE_INSERTS As UInteger = &H200
            Const FORMAT_MESSAGE_FROM_SYSTEM As UInteger = &H1000
            Dim sb As StringBuilder = New StringBuilder(1024)
            FormatMessage(
                FORMAT_MESSAGE_FROM_SYSTEM Or FORMAT_MESSAGE_IGNORE_INSERTS,
                IntPtr.Zero,
                win32error,
                0,
                sb,
                1024,
                IntPtr.Zero)
            Dim errorString As String = sb.ToString()
            Return errorString
        End Function

        Public Shared Function GetCurrentTimeString() As String
            Dim now As DateTime = DateTime.Now
            Return now.ToString("yyyy:MM:dd hh:mm:ss:fff")
        End Function

        ''' <summary>
        ''' Cancel a job started by StartJob().
        ''' do not throw any exceptions.
        ''' </summary>
        Public Shared Sub CancelJob(bidiSpl As BidiSplWrap, printerJobID As Integer, errorCode As Integer)
            Dim xmlFormat As String = strings.PRINTER_ACTION_XML
            Dim input As String = String.Format(xmlFormat, CInt(Actions.Cancel), printerJobID, errorCode)
            bidiSpl.SetPrinterData(strings.PRINTER_ACTION, input)
        End Sub

        ''' <summary>
        '''
        ''' Issue an IBidi StartJob with xml markup to start an IBidi print job.
        '''
        ''' We do this to have the printer check for supplies, or, just to obtain the
        ''' printerJobID for job completion polling later.
        '''
        ''' If the printer is out of supplies, throw an exception.
        '''
        ''' </summary>
        Public Shared Function StartJob(bidiSpl As BidiSplWrap, checkSupplies As Boolean, hopperID As String) As Integer

            Const pollSleepMilliseconds As Integer = 2000

            Dim printerJobID As Integer = 0

            Do
                Dim printerOptionsXML As String = bidiSpl.GetPrinterData(strings.PRINTER_OPTIONS2)
                Dim printerOptionsValues As PrinterOptionsValues = Util.ParsePrinterOptionsXML(printerOptionsXML)

                Dim checkPrintRibbon As Integer = 0
                Dim checkEmbossSupplies As Integer = 0
                If checkSupplies Then
                    checkPrintRibbon = If((printerOptionsValues._printHead = "Installed"), 1, 0)
                    checkEmbossSupplies = If(printerOptionsValues._moduleEmbosser = "None", 0, 1)
                End If

                Dim startJobXml As String = String.Format(strings.STARTJOB_XML, hopperID, checkPrintRibbon, checkEmbossSupplies)

                Dim printerStatusXML As String = bidiSpl.SetPrinterData(strings.STARTJOB, startJobXml)
                Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)

                If 506 = printerStatusValues._errorCode Then

                    ' Printer cannot accept another job as it is busy. Try again.
                    Console.WriteLine("StartJob(): {0} Trying again.", printerStatusValues._errorString)

                    ' let the current card process in the printer:
                    Thread.Sleep(pollSleepMilliseconds)
                ElseIf 0 <> printerStatusValues._errorCode Then
                    Dim message As String = String.Format("StartJob(): {0} {1}", printerStatusValues._errorCode, printerStatusValues._errorString)
                    Throw New BidiException(message, printerStatusValues._printerJobID, printerStatusValues._errorCode)
                Else
                    Console.WriteLine("Printer job ID: {0} started.", printerStatusValues._printerJobID)
                    printerJobID = printerStatusValues._printerJobID

                End If
            Loop While 0 = printerJobID

            Return printerJobID
        End Function

        ''' <summary>
        ''' repeatedly check job status until the job is complete - failed or not.
        ''' </summary>
        Public Shared Sub PollForJobCompletion(bidiSpl As BidiSplWrap, printerJobID As Integer)

            Const pollSleepMilliseconds As Integer = 2000

            While True

                Dim printerStatusXML As String = bidiSpl.GetPrinterData(strings.PRINTER_MESSAGES)
                Dim printerStatusValues As PrinterStatusValues = Util.ParsePrinterStatusXML(printerStatusXML)

                If (0 <> printerStatusValues._errorCode) AndAlso (printerJobID = printerStatusValues._printerJobID) Then
                    Dim message As String = String.Format("{0} severity: {1}", printerStatusValues._errorString, printerStatusValues._errorSeverity)
                    Throw New BidiException(message, printerStatusValues._printerJobID, printerStatusValues._errorCode)
                End If

                Dim jobStatusRequest As String = strings.JOB_STATUS_XML
                Dim jobInfoXML As String = String.Format(jobStatusRequest, printerJobID)
                Dim jobStatusXML As String = bidiSpl.GetPrinterData(strings.JOB_STATUS, jobInfoXML)
                Dim jobStatusValues As JobStatusValues = Util.ParseJobStatusXML(jobStatusXML)

                Console.WriteLine("Printer job ID: {0} {1}", printerJobID, jobStatusValues._jobState)

                If strings.JOB_ACTIVE <> jobStatusValues._jobState Then
                    Return
                End If

                Thread.Sleep(pollSleepMilliseconds)
            End While
        End Sub
    End Class

End Namespace