﻿''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright (c) Datacard Corporation.  All Rights Reserved.
'
' XPS Printer SDK: vb.net 'printer_control' sample
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Imports System.IO
Imports dxp01sdk

Public Enum Commands
    undefined
    cancelAllJobs
    restart
    resetCardCounts
End Enum

Public Class CommandLineOptions
    Public Sub New()
        command = Commands.undefined
    End Sub
    Public printerName As String
    Public command As Commands
End Class

Class control
    Private Shared Sub Usage()
        Dim thisExeName As String = Path.GetFileName(Environment.GetCommandLineArgs()(0))
        Console.WriteLine()
        Console.WriteLine(thisExeName & " controls the printer through the printer driver.")
        Console.WriteLine()
        Console.WriteLine(thisExeName & " -n <printername> <command>")
        Console.WriteLine("  -n <printername>. Required. Try -n ""XPS Card Printer"".")
        Console.WriteLine("  -c cancel all jobs: Send a command to the printer to clear any print job")
        Console.WriteLine("                    from the printer. This does not delete spooled Windows")
        Console.WriteLine("                    print jobs.")
        Console.WriteLine("  -r restart the printer.")
        Console.WriteLine("  -e reset all card counts.")
        Console.WriteLine()
        Environment.Exit(-1)
    End Sub

    Private Shared Function GetCommandlineOptions(args As String()) As CommandLineOptions
        Dim commandLineOptions As New CommandLineOptions()
        Dim arguments As New CommandLine.Utility.Arguments(args)

        If String.IsNullOrEmpty(arguments("n")) Then
            Usage()
        End If

        ' check for -n with no printer name:
        Dim boolVal As Boolean = False
        If Boolean.TryParse(arguments("n"), boolVal) Then
            Usage()
        End If
        commandLineOptions.printerName = arguments("n")

        If Not String.IsNullOrEmpty(arguments("c")) Then
            commandLineOptions.command = Commands.cancelAllJobs
        End If

        If Not String.IsNullOrEmpty(arguments("r")) Then
            commandLineOptions.command = Commands.restart
        End If

        If Not String.IsNullOrEmpty(arguments("e")) Then
            commandLineOptions.command = Commands.resetCardCounts
        End If

        If Commands.undefined = commandLineOptions.command Then
            Usage()
        End If

        Return commandLineOptions
    End Function
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Main()
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Shared Sub Main(args As String())

        Dim commandLineOptions As CommandLineOptions = GetCommandlineOptions(args)

        Dim bidiSpl As New BidiSplWrap()
        bidiSpl.BindDevice(commandLineOptions.printerName)

        Select Case commandLineOptions.command
            Case Commands.cancelAllJobs
                ' using zero for the printerJobID and the errorCode causes all
                ' printerJobs to be canceled:
                Util.CancelJob(bidiSpl, 0, 0)
                Console.WriteLine("all jobs canceled for '{0}'", commandLineOptions.printerName)
                Exit Select

            Case Commands.restart
                Dim result As String = bidiSpl.SetPrinterData(strings.RESTART_PRINTER)
                Console.WriteLine("command issued to restart the printer.")
                Exit Select

            Case Commands.resetCardCounts
                Dim result As String = bidiSpl.SetPrinterData(strings.RESET_CARD_COUNTS)
                Console.WriteLine("command issued to reset the printer's card counts.")
                Exit Select

            Case Else
                Console.WriteLine("unexpected commandline option.")
                Exit Select

        End Select

        bidiSpl.UnbindDevice()
    End Sub
End Class
